from collections import MutableMapping
import os
import zipfile

import pytest

from zict import Zip

@pytest.yield_fixture
def fn():
    filename = '.tmp.zip'
    if os.path.exists(filename):
        os.remove(filename)

    yield filename

    if os.path.exists(filename):
        os.remove(filename)


def test_simple(fn):
    z = Zip(fn)
    assert isinstance(z, MutableMapping)
    assert not z

    assert list(z) == list(z.keys()) == []
    assert list(z.values()) == []
    assert list(z.items()) == []

    z['x'] = b'123'
    assert list(z) == list(z.keys()) == ['x']
    assert list(z.values()) == [b'123']
    assert list(z.items()) == [('x', b'123')]
    assert z['x'] == b'123'

    z.flush()
    zz = zipfile.ZipFile(fn, mode='r')
    assert zz.read('x') == b'123'

    z['y'] = b'456'
    assert z['y'] == b'456'


def test_setitem_typeerror(fn):
    z = Zip(fn)
    with pytest.raises(TypeError):
        z['x'] = 123


def test_contextmanager(fn):
    with Zip(fn) as z:
        z['x'] = b'123'

    zz = zipfile.ZipFile(fn, mode='r')
    assert zz.read('x') == b'123'


def test_missing_key(fn):
    z = Zip(fn)

    with pytest.raises(KeyError):
        z['x']


def test_close(fn):
    z = Zip(fn)

    z['x'] = b'123'
    z.close()

    zz = zipfile.ZipFile(fn, mode='r')
    assert zz.read('x') == b'123'

    with pytest.raises(IOError):
        z['y'] = b'123'


def test_bytearray(fn):
    data = bytearray(b'123')
    with Zip(fn) as z:
        z['x'] = data

    with Zip(fn) as z:
        assert z['x'] == b'123'



from __future__ import absolute_import, division, print_function

from zict import LRU
from . import utils_test


def test_simple():
    d = dict()
    lru = LRU(2, d)

    lru['x'] = 1
    lru['y'] = 2

    assert lru['x'] == 1
    assert lru['y'] == 2
    assert d == {'x': 1, 'y': 2}

    lru['z'] = 3
    assert len(d) == 2
    assert len(lru) == 2
    assert 'z' in d
    assert 'z' in lru
    assert 'x' not in d
    assert 'y' in d

    del lru['y']
    assert 'y' not in d
    assert 'y' not in lru

    lru['a'] = 5
    assert set(lru.keys()) == set(['z', 'a'])


def test_str():
    d = dict()
    lru = LRU(2, d)

    lru['x'] = 1
    lru['y'] = 2

    assert str(lru.total_weight) in str(lru)
    assert str(lru.total_weight) in repr(lru)
    assert str(lru.n) in str(lru)
    assert str(lru.n) in repr(lru)
    assert 'dict' in str(lru)
    assert 'dict' in repr(lru)


def test_mapping():
    """
    Test mapping interface for LRU().
    """
    d = {}
    # 100 is more than the max length when running check_mapping()
    lru = LRU(100, d)
    utils_test.check_mapping(lru)
    utils_test.check_closing(lru)


def test_overwrite():
    d = dict()
    lru = LRU(2, d)

    lru['x'] = 1
    lru['y'] = 2
    lru['y'] = 3

    assert set(lru) == {'x', 'y'}

    lru.update({'y': 4})

    assert set(lru) == {'x', 'y'}


def test_callbacks():
    count = [0]
    def cb(k, v):
        count[0] += 1

    L = list()
    d = dict()
    lru = LRU(2, d, on_evict=[lambda k, v: L.append((k, v)), cb])

    lru['x'] = 1
    lru['y'] = 2
    lru['z'] = 3

    assert L == [('x', 1)]
    assert count[0] == len(L)


def test_weight():
    d = dict()
    weight = lambda k, v: v
    lru = LRU(10, d, weight=weight)

    lru['x'] = 5
    assert lru.total_weight == 5

    lru['y'] = 4
    assert lru.total_weight == 9

    lru['z'] = 3
    assert d == {'y': 4, 'z': 3}
    assert lru.total_weight == 7

    del lru['z']
    assert lru.total_weight == 4

    lru['a'] = 10000
    assert 'a' not in lru
    assert d == {'y': 4}



from __future__ import absolute_import, division, print_function

import sys

from zict import Sieve
from . import utils_test


def test_simple():
    a = {}
    b = {}
    c = {}
    def selector(k, v):
        return len(v) % 3
    mappings = {0: a, 1: b, 2: c}

    d = Sieve(mappings, selector)
    assert len(d) == 0

    d['u'] = b'the'
    d['v'] = b'big'
    d['w'] = b'brown'
    d['x'] = b'fox'
    d['y'] = b'jumps'
    d['z'] = b'over'

    assert d['u'] == b'the'
    assert d['v'] == b'big'
    assert len(d) == 6

    assert sorted(d) == ['u', 'v', 'w', 'x', 'y', 'z']
    assert sorted(d.keys()) == ['u', 'v', 'w', 'x', 'y', 'z']
    assert sorted(d.values()) == sorted([b'the', b'big', b'brown',
                                         b'fox', b'jumps', b'over'])

    assert a == {'u': b'the', 'v': b'big', 'x': b'fox'}
    assert b == {'z': b'over'}
    assert c == {'w': b'brown', 'y': b'jumps'}

    # Changing existing keys can move values from one mapping to another.
    d['w'] = b'lazy'
    d['x'] = b'dog'
    assert d['w'] == b'lazy'
    assert d['x'] == b'dog'
    assert len(d) == 6
    assert sorted(d.values()) == sorted([b'the', b'big', b'lazy',
                                         b'dog', b'jumps', b'over'])

    assert a == {'u': b'the', 'v': b'big', 'x': b'dog'}
    assert b == {'w': b'lazy', 'z': b'over'}
    assert c == {'y': b'jumps'}

    del d['v']
    del d['w']
    assert len(d) == 4
    assert 'v' not in d
    assert 'w' not in d
    assert sorted(d.values()) == sorted([b'the', b'dog', b'jumps', b'over'])


def test_mapping():
    """
    Test mapping interface for Sieve().
    """
    a = {}
    b = {}
    def selector(key, value):
        return sum(bytearray(value)) & 1
    mappings = {0: a, 1: b}
    z = Sieve(mappings, selector)
    utils_test.check_mapping(z)
    utils_test.check_closing(z)



from __future__ import absolute_import, division, print_function

from zict import Buffer
from . import utils_test


def test_simple():
    a = dict()
    b = dict()
    buff = Buffer(a, b, n=10, weight=lambda k, v: v)

    buff['x'] = 1
    buff['y'] = 2

    assert buff['x'] == 1
    assert buff['y'] == 2
    assert a == {'x': 1, 'y': 2}
    assert buff.fast.total_weight == 3

    buff['z'] = 8
    assert a == {'y': 2, 'z': 8}
    assert b == {'x': 1}

    assert buff['x'] == 1
    assert a == {'x': 1, 'z': 8}
    assert b == {'y': 2}

    assert 'x' in buff
    assert 'y' in buff
    assert 'missing' not in buff

    buff['y'] = 1
    assert a == {'x': 1, 'y': 1, 'z': 8}
    assert buff.fast.total_weight == 10
    assert b == {}

    del buff['z']
    assert a == {'x': 1, 'y': 1}
    assert buff.fast.total_weight == 2
    assert b == {}

    del buff['y']
    assert a == {'x': 1}
    assert buff.fast.total_weight == 1
    assert b == {}

    assert 'y' not in buff

    buff['a'] = 5
    assert set(buff) == set(buff.keys()) == {'a', 'x'}

    fast_keys = set(buff.fast)
    slow_keys = set(buff.slow)
    assert not (fast_keys & slow_keys)
    assert fast_keys | slow_keys == set(buff)

    # Overweight element stays in slow mapping
    buff['b'] = 1000
    assert 'b' in buff.slow
    assert set(buff.fast) == fast_keys
    assert set(buff.slow) == {'b'} | slow_keys
    assert 'b' in buff
    assert buff['b'] == 1000


def test_mapping():
    """
    Test mapping interface for Buffer().
    """
    a = {}
    b = {}
    buff = Buffer(a, b, n=2)
    utils_test.check_mapping(buff)
    utils_test.check_closing(buff)


def test_callbacks():
    f2s = []
    def f2s_cb(k, v):
        f2s.append(k)

    s2f = []
    def s2f_cb(k, v):
        s2f.append(k)

    a = dict()
    b = dict()
    buff = Buffer(a, b, n=10, weight=lambda k, v: v,
                  fast_to_slow_callbacks=f2s_cb,
                  slow_to_fast_callbacks=s2f_cb)

    buff['x'] = 1
    buff['y'] = 2

    assert buff['x'] == 1
    assert buff['y'] == 2
    assert not f2s
    assert not s2f

    buff['z'] = 8

    assert f2s == ['x']
    assert s2f == []
    buff['z']

    assert f2s == ['x']
    assert s2f == []

    buff['x']
    assert f2s == ['x', 'y']
    assert s2f == ['x']



from __future__ import absolute_import, division, print_function

from zict import Func
from . import utils_test


def inc(x):
    return x + 1

def dec(x):
    return x - 1

def rotl(x):
    return x[1:] + x[:1]

def rotr(x):
    return x[-1:] + x[:-1]


def test_simple():
    d = dict()
    f = Func(inc, dec, d)
    f['x'] = 10
    assert f['x'] == 10
    assert d['x'] == 11

    assert 'x' in f
    assert list(f) == ['x']
    assert list(f.values()) == [10]
    assert list(f.items()) == [('x', 10)]

    assert all(s in str(f) for s in ['inc', 'dec', 'x', 'Func'])
    assert all(s in repr(f) for s in ['inc', 'dec', 'x', 'Func'])

    del f['x']
    assert 'x' not in d


def test_mapping():
    """
    Test mapping interface for Func().
    """
    d = {}
    z = Func(rotl, rotr, d)
    utils_test.check_mapping(z)
    utils_test.check_closing(z)



#!/usr/bin/env python

import os
from setuptools import setup

setup(name='zict',
      version='0.1.2',
      description='Mutable mapping tools',
      url='http://github.com/dask/zict/',
      maintainer='Matthew Rocklin',
      maintainer_email='mrocklin@gmail.com',
      license='BSD',
      keywords='mutable mapping dict',
      packages=['zict'],
      install_requires=[open('requirements.txt').read().strip().split('\n')],
      long_description=(open('README.rst').read() if os.path.exists('README.rst')
                        else ''),
      zip_safe=False)
